import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera import *
from .processing_config import *
from .camera_config import *


class KeaCamera(Camera):
    """
    The main interface to the kea camera
    """
    def __del__(self):
        if self._ptr:
            lib.tof_kea_camera_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, processing_config: Optional[ProcessingConfig] = None, serial: Optional[str] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new kea_camera class
        ----------

        Construct the kea camera

        Parameters
        ----------
        processing_config : ProcessingConfig
                The processing config
        serial : str
                The serial number of the camera to connect to, if the camera is
                available over both usb and gige it will be connected over usb. If
                an empty string is supplied the camera will connect to the first usb
                camera, if no usb camera is connected it will connect to the first
                gige camera that replies (this reply time is random, so you might
                connect to a different camera each time if you've got multiple
                cameras connected to your network). If no camera is found with the
                given serial number a nullptr and error is returned.
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif processing_config != None and serial != None:
            self._ptr = lib.tof_kea_camera_new(processing_config._ptr, serial.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def onCameraProcessingCapable(self) -> bool:
        """
        Check whether this kea camera can do on camera processing

        Returns
        ----------
        bool
            On camera processing capable
        """
        return lib.tof_kea_camera_on_camera_processing_capable(self._ptr, get_error())


    def getOnCameraProcessing(self) -> bool:
        """
        Get on camera processing

        Returns
        ----------
        bool
            True on camera processing, false on host processing
        """
        return lib.tof_kea_camera_get_on_camera_processing(self._ptr, get_error())


    def setOnCameraProcessing(self, on_camera_processing: bool) -> None:
        """
        Set on camera processing

        Parameters
        ----------
        on_camera_processing : bool
            True on camera processing, false on host processing
        """
        return lib.tof_kea_camera_set_on_camera_processing(self._ptr, on_camera_processing, get_error())


    def getDelay(self) -> int:
        """
        Get delay between network packets

        Returns
        ----------
        int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_kea_camera_get_delay(self._ptr, get_error())


    def setDelay(self, delay: int) -> None:
        """
        Set delay between network packets

        Parameters
        ----------
        delay : int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_kea_camera_set_delay(self._ptr, delay, get_error())


    def getPacketSize(self) -> int:
        """
        Get maximum size of each network packet transmitted

        Returns
        ----------
        int
            Packet size in bytes
        """
        return lib.tof_kea_camera_get_packet_size(self._ptr, get_error())


    def setPacketSize(self, size: int) -> None:
        """
        Set maximum size of each network packet transmitted

        Parameters
        ----------
        size : int
            Packet size in bytes
        """
        return lib.tof_kea_camera_set_packet_size(self._ptr, size, get_error())


    def setCameraConfig(self, camera_config: CameraConfig) -> None:
        """
        Set the camera config

        Parameters
        ----------
        camera_config : CameraConfig
            Camera config
        """
        return lib.tof_kea_camera_set_camera_config(self._ptr, camera_config._ptr, get_error())


    def setConfigurations(self, camera_config: CameraConfig, processing_config: ProcessingConfig) -> None:
        """
        Set both the camera and processing config

        Parameters
        ----------
        camera_config : CameraConfig
            Camera config
        processing_config : ProcessingConfig
            Processing config
        """
        return lib.tof_kea_camera_set_configurations(self._ptr, camera_config._ptr, processing_config._ptr, get_error())


    def version(self) -> str:
        """
        Get the tof library version running on the camera

        Returns
        ----------
        str
            Version
        """
        return lib.tof_kea_camera_version(self._ptr, get_error()).decode("utf-8")


