#include "cxxopts.hpp"
#include "print.hpp"

struct Options {
  std::string serial;
  std::string file_name;
  bool download;
  bool calibration;
};

template <typename Camera>
void handle_file_exchange(Camera &cam, const Options &options) {
  std::cout << "Connected to camera " << options.serial << "\n";

  if (options.calibration) {
    if (options.download) {
      std::cout << "Downloading calibration file to " << options.file_name
                << "\n";
      tof::Calibration cal = cam.download_calibration();
      cal.write(options.file_name);
    } else {
#ifdef TOF_INTERNAL
      std::cout << "Uploading " << options.file_name << "\n";
      tof::Calibration cal(options.file_name);
      cam.upload_calibration(cal);
#else
      std::cout << "Uploading calibration file to camera is not supported\n";
#endif
    }
  } else {
    if (options.download) {
      std::cout << "Downloading configuration file to " << options.file_name
                << "\n";
      tof::CameraConfig config = cam.download_configuration();
      config.write(options.file_name);
    } else {
      std::cout << "Uploading " << options.file_name << "\n";
      tof::CameraConfig config(options.file_name);
      cam.upload_configuration(config);
    }
  }
}

int main(int argc, char **argv) {
  cxxopts::Options arg_options("ToF file_type Exchange",
                               "Upload/Download files from chronoptics camera");

  Options options{};
  uint16_t port = 0;
  // clang-format off
  arg_options.add_options()
    ("h, help", "Help")
    ("list", "List cameras on network")
    ("s, serial", "Serial number of camera to connect to", cxxopts::value<std::string>(options.serial))
    ("p, port", "Port to talk on", cxxopts::value<uint16_t>(port))
    ("f, file", "The file name", cxxopts::value<std::string>(options.file_name))
    ("d, download", "Download")
    ("u, upload", "Upload")
    ("config", "Exchange config")
    ("calibration", "Exchange calibration")
  ;
  // clang-format on

  auto result = arg_options.parse(argc, argv);

  if (result.count("h") || result.count("help")) {
    std::cout << arg_options.help() << std::endl;
    return 0;
  }

  if (result.count("list")) {
    print_devices();
    return 0;
  }

  if (result.count("d") + result.count("u") != 1) {
    std::cout << "Specify one -u or -d\n";
    return -1;
  }

  if (!result.count("serial")) {
    std::cout << "Please specify camera serial\n";
    return -1;
  }

  options.download = result.count("d") > 0;

  if (result.count("config") + result.count("calibration") != 1) {
    std::cout
        << "Specify one of the following files: --config, --calibration\n";
    return -1;
  }

  options.calibration = result.count("calibration") > 0;

  bool use_usb = true;
  try {
    tof::usb_device_find(options.serial);
    std::cout << "Using usb interface\n";
  } catch (std::exception &e) {
    use_usb = false;
    std::cout << "Using gige interface\n";
  }

  try {
    if (use_usb) {
      auto device = tof::usb_device_find(options.serial);
      tof::UsbInterface cam(device);
      if (cam.test_stream_speed() < 1.0) {
        std::cout << "USB speed to low\n";
        return -1;
      }

      handle_file_exchange(cam, options);
    } else {
      tof::GigeInterface cam(port);
      auto msg = cam.find(options.serial);

      cam.connect(msg);
      if (!cam.test_stream_packet_size()) {
        std::cout << "Failed test packet size\n";
        return -1;
      }

      handle_file_exchange(cam, options);
    }

    std::cout << "Success\n";
  } catch (std::exception &e) {
    std::cout << "Failed with " << e.what() << "\n";
  }

  return 0;
}
